@@ -2,7 +2,7 @@
 # utils module - common functions for reportbug UIs
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2010 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2011 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -19,10 +19,6 @@
 ##  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ##  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 ##  SOFTWARE.
-#
-# Version ##VERSION##; see changelog for revision history
-#
-# $Id: reportbug.py,v 1.35.2.24 2008-04-18 05:38:28 lawrencc Exp $
 
 import sys
 import os
@@ -30,7 +26,7 @@ import re
 import platform
 try:
     import pwd
-    from tempfiles import TempFile, tempfile_prefix
+    from tempfiles import TempFile, tempfile_prefix, cleanup_temp_file
 except ImportError, e:
     if platform.system() == 'Windows':
         pass
@@ -43,6 +39,7 @@ import rfc822
 import socket
 import subprocess
 
+from urlutils import open_url
 from string import ascii_letters, digits
 
 # Paths for dpkg
@@ -70,17 +67,33 @@ for mode in MODELIST:
 del mode
 
 # moved here since it needs the MODE_* vars to be defined
-import debianbts
-# it needs to be imported after debianbts
+import debbugs
+# it needs to be imported after debbugs
 import ui.text_ui as ui
 
 from reportbug.ui import AVAILABLE_UIS
 
-NEWBIELINE = '*** Please type your report below this line ***'
+NEWBIELINE = """Dear Maintainer,
+*** Please consider answering these questions, where appropriate ***
+
+   * What led up to the situation?
+   * What exactly did you do (or not do) that was effective (or
+     ineffective)?
+   * What was the outcome of this action?
+   * What outcome did you expect instead?
+
+*** End of the template - remove these lines ***"""
 
 fhs_directories = ['/', '/usr', '/usr/share', '/var', '/usr/X11R6',
                    '/usr/man', '/usr/doc', '/usr/bin']
 
+# A map between suites and distributions names
+SUITES2DISTS = {'lenny': 'oldstable',
+                'squeeze': 'stable',
+                'wheezy': 'testing',
+                'sid': 'unstable',
+                'experimental': 'experimental'}
+
 def realpath(filename):
     filename = os.path.abspath(filename)
 
@@ -308,9 +321,10 @@ def get_package_status(package, avail=False):
     descre = re.compile('Description: ')
     fullre = re.compile(' ')
     srcre = re.compile('Source: ')
+    sectionre = re.compile('Section: ')
 
     pkgversion = pkgavail = maintainer = status = origin = None
-    bugs = vendor = priority = desc = src_name = None
+    bugs = vendor = priority = desc = src_name = section = None
     conffiles = []
     fulldesc = []
     depends = []
@@ -384,6 +398,8 @@ def get_package_status(package, avail=False):
         elif srcre.match(line):
             crud, src_name = line.split(": ", 1)
             src_name = src_name.split()[0]
+        elif sectionre.match(line):
+            crud, section = line.split(": ", 1)
         elif desc and line[0]==' ':
             fulldesc.append(line)
 
@@ -394,12 +410,12 @@ def get_package_status(package, avail=False):
 
     reportinfo = None
     if bugs:
-        reportinfo = debianbts.parse_bts_url(bugs)
+        reportinfo = debbugs.parse_bts_url(bugs)
     elif origin:
-        if debianbts.SYSTEMS.has_key(origin):
-            vendor = debianbts.SYSTEMS[origin]['name']
-            reportinfo = (debianbts.SYSTEMS[origin].get('type', 'debbugs'),
-                          debianbts.SYSTEMS[origin]['btsroot'])
+        if debbugs.SYSTEMS.has_key(origin):
+            vendor = debbugs.SYSTEMS[origin]['name']
+            reportinfo = (debbugs.SYSTEMS[origin].get('type', 'debbugs'),
+                          debbugs.SYSTEMS[origin]['btsroot'])
         else:
             vendor = origin.capitalize()
     else:
@@ -408,7 +424,8 @@ def get_package_status(package, avail=False):
     info = (pkgversion, pkgavail, tuple(depends), tuple(recommends),
             tuple(conffiles),
             maintainer, installed, origin, vendor, reportinfo, priority,
-            desc, src_name, os.linesep.join(fulldesc), state, tuple(suggests))
+            desc, src_name, os.linesep.join(fulldesc), state, tuple(suggests),
+            section)
 
     if not avail:
         statuscache[package] = info
@@ -564,7 +581,10 @@ def get_package_info(packages, skip_notfound=False):
                 stat = statob.search(p).group('stat')
                 sinfo = stat.split()
                 stat = sinfo[0][0] + sinfo[2][0]
-                if stat[1] != 'i':
+                # check if the package is installed, and in that case, retrieve
+                # its information; if the first char is not 'i' (installed) or
+                # the second is 'n' (not-installed), then skip data retrieval
+                if stat[0] != 'i' or stat[1] == 'n':
                     continue
 
                 if m.group('hdr') == 'Provides':
@@ -626,15 +646,30 @@ def get_dependency_info(package, depends, rel="depends on"):
 
     deplist = packs.values()
     deplist.sort()
-    maxlen = max([len(x[2]) for x in deplist] + [10])
 
+    deplist2 = []
+    # extract the info we need, also add provides where it fits
     for (pack, status, vers, desc, provides) in deplist:
         if provides:
             pack += ' [' + provides + ']'
+        deplist2.append((pack, vers))
+    deplist = deplist2
 
-        packstuff = '%-*.*s %s' % (39-maxlen, 39-maxlen, pack, vers)
+    # now we can compute the max possible value for each column, that can be the
+    # max of all its values, or the space left from the other column; this way,
+    # the sum of the 2 fields is never > 73 (hence the resulting line is <80
+    # columns)
+    maxp = max([len(x[0]) for x in deplist])
+    maxv = max([len(x[1]) for x in deplist])
+    widthp = min(maxp, 73-maxv)
+    widthv = min(maxv, 73-widthp)
 
-        info = '%-3.3s %-40.40s %-.34s\n' % (status, packstuff, desc)
+    for (pack, vers) in deplist:
+        # we format the string specifying to align it in a field of a given
+        # dimension (the first {width*}) but also limit its size (the second
+        # {width*}
+        info = '{0:3.3} {1:{widthp}.{widthp}}  {2:{widthv}.{widthv}}\n'.format(
+            status, pack, vers, widthp=widthp, widthv=widthv)
         depinfo += info
 
     return depinfo
@@ -666,14 +701,14 @@ def get_changed_config_files(conffiles, nocompress=False):
 
     return confinfo, changed
 
-DISTORDER = ['stable', 'testing', 'unstable', 'experimental']
+DISTORDER = ['oldstable', 'stable', 'testing', 'unstable', 'experimental']
 
 def get_debian_release_info():
     debvers = debinfo = verfile = warn = ''
     dists = []
     output = commands.getoutput('apt-cache policy 2>/dev/null')
     if output:
-        mre = re.compile('\s+(\d+)\s+.*$\s+release\s.*o=(Ubuntu|Debian),a=([^,]+),', re.MULTILINE)
+        mre = re.compile('\s+(\d+)\s+.*$\s+release\s.*o=(Ubuntu|Debian|Debian Ports),a=([^,]+),', re.MULTILINE)
         found = {}
         ## XXX: When Python 2.4 rolls around, rewrite this
         for match in mre.finditer(output):
@@ -714,10 +749,10 @@ def get_debian_release_info():
     return debinfo
 
 def lsb_release_info():
-    return commands.getoutput('lsb_release -a 2>/dev/null')
+    return commands.getoutput('lsb_release -a 2>/dev/null') + '\n'
 
 def get_arch():
-    arch = commands.getoutput('COLUMNS=79 dpkg --print-installation-architecture 2>/dev/null')
+    arch = commands.getoutput('COLUMNS=79 dpkg --print-architecture 2>/dev/null')
     if not arch:
         un = os.uname()
         arch = un[4]
@@ -728,7 +763,7 @@ def get_arch():
 
 def generate_blank_report(package, pkgversion, severity, justification,
                           depinfo, confinfo, foundfile='', incfiles='',
-                          system='debian', exinfo=0, type=None, klass='',
+                          system='debian', exinfo=None, type=None, klass='',
                           subject='', tags='', body='', mode=MODE_EXPERT,
                           pseudos=None, debsumsoutput=None, issource=False):
     # For now...
@@ -736,10 +771,12 @@ def generate_blank_report(package, pkgversion, severity, justification,
 
     sysinfo = (package not in ('wnpp', 'ftp.debian.org'))
 
+    # followup is where bugreport expects the notification of the bug reportbug
+    # to follow-up, but reportbug pass this information with 'exinfo'
     rep = bugreport.bugreport(package, version=pkgversion, severity=severity,
                               justification=justification, filename=foundfile,
                               mode=mode, subject=subject, tags=tags, body=body,
-                              pseudoheaders=pseudos, exinfo=exinfo, type=type,
+                              pseudoheaders=pseudos, followup=exinfo, type=type,
                               system=system, depinfo=depinfo, sysinfo=sysinfo,
                               confinfo=confinfo, incfiles=incfiles,
                               debsumsoutput=debsumsoutput, issource=issource)
@@ -756,7 +793,7 @@ def get_cpu_cores():
     for line in fob:
         if line.startswith('processor'):
             cpucount += 1
-	#Alpha plateform
+	#Alpha platform
 	if line.startswith('cpus detected'):
 	    cpucount = int(line.split()[-1])
     fob.close()
@@ -766,7 +803,7 @@ def get_cpu_cores():
 class our_lex(shlex.shlex):
     def get_token(self):
         token = shlex.shlex.get_token(self)
-        if not len(token): return token
+        if token is None or not len(token): return token
         if (token[0] == token[-1]) and token[0] in self.quotes:
             token = token[1:-1]
         return token
@@ -780,7 +817,7 @@ CONFIG_ARGS = (
     'sign', 'nocc', 'nocompress', 'dontquery', 'noconf', 'mirrors', 'keyid',
     'headers', 'interface', 'template', 'mode', 'check_available', 'query_src',
     'printonly', 'offline', 'check_uid', 'smtptls', 'smtpuser', 'smtppasswd',
-    'paranoid')
+    'paranoid', 'mbox_reader_cmd', 'max_attachment_size')
 
 class Mua:
     command = ""
@@ -818,6 +855,7 @@ MUA = {
     'mutt' : Mua('mutt -H'),
     'mh' : Mua('/usr/bin/mh/comp -use -file'),
     'gnus' : Gnus(),
+    'claws-mail' : Mua('claws-mail --compose-read'),
     }
 MUA['nmh'] = MUA['mh']
 
@@ -826,6 +864,7 @@ MUAVERSION = {
     MUA['mutt'] : 'mutt -v',
     MUA['mh'] : '/usr/bin/mh/comp -use -file',
     MUA['gnus'] : 'emacs --version',
+    MUA['claws-mail'] : 'claws-mail --version',
     }
 
 def mua_is_supported(mua):
@@ -886,19 +925,20 @@ def parse_config_files():
     for filename in FILES:
         if os.path.exists(filename):
             try:
-                lex = our_lex(file(filename))
+                lex = our_lex(file(filename), posix=True)
             except IOError, msg:
                 continue
 
             lex.wordchars = lex.wordchars + '-.@/:<>'
 
-            token = lex.get_token().lower()
+            token = lex.get_token()
             while token:
+                token = token.lower()
                 if token in ('quiet', 'maintonly', 'submit'):
                     args['sendto'] = token
                 elif token == 'severity':
                     token = lex.get_token().lower()
-                    if token in debianbts.SEVERITIES.keys():
+                    if token in debbugs.SEVERITIES.keys():
                         args['severity'] = token
                 elif token == 'header':
                     args['headers'] = args.get('headers', []) + \
@@ -915,7 +955,8 @@ def parse_config_files():
                     args[token] = True
                 elif token in ('email', 'realname', 'replyto', 'http_proxy',
                                'smtphost', 'editor', 'mua', 'mta', 'smtpuser',
-                               'smtppasswd', 'justification', 'keyid'):
+                               'smtppasswd', 'justification', 'keyid',
+                               'mbox_reader_cmd'):
                     bit = lex.get_token()
                     args[token] = bit.decode('utf-8', 'replace')
                 elif token in ('no-smtptls', 'smtptls'):
@@ -938,7 +979,7 @@ def parse_config_files():
                         args[token] = arg
                 elif token == 'bts':
                     token = lex.get_token().lower()
-                    if token in debianbts.SYSTEMS.keys():
+                    if token in debbugs.SYSTEMS.keys():
                         args['bts'] = token
                 elif token == 'mirror':
                     args['mirrors'] = args.get('mirrors', []) + \
@@ -961,10 +1002,16 @@ def parse_config_files():
                     args['check_uid'] = (token == 'check-uid')
                 elif token in ('paranoid', 'no-paranoid'):
                     args['paranoid'] = (token == 'paranoid')
+                elif token == 'max_attachment_size':
+                    arg = lex.get_token()
+                    args['max_attachment_size'] = int(arg)
+                elif token == 'envelopefrom':
+                    token = lex.get_token().lower()
+                    args['envelopefrom'] = token
                 else:
                     sys.stderr.write('Unrecognized token: %s\n' % token)
 
-                token = lex.get_token().lower()
+                token = lex.get_token()
 
     return args
 
@@ -1075,3 +1122,78 @@ def cleanup_msg(dmessage, headers, pseudos, type):
                 ph += ['%s: %s' % (header, ph2[header])]
 
     return message, newheaders, ph
+
+def launch_mbox_reader(cmd, url, http_proxy, timeout):
+    """Runs the command specified by cmd passing the mbox file
+    downloaded from url as a parameter. If cmd is None or fails, then
+    fallback to mail program."""
+    mbox = open_url(url, http_proxy, timeout)
+    if mbox is None:
+        return
+    (fd, fname) = TempFile()
+    try:
+        for line in mbox:
+            fd.write(line)
+        fd.close()
+        if cmd is not None:
+            try:
+                cmd = cmd % fname
+            except TypeError:
+                cmd = "%s %s" % (cmd, fname)
+            error = os.system(cmd)
+            if not error:
+                return
+        #fallback
+        os.system('mail -f ' + fname)
+    finally:
+        os.unlink(fname)
+
+def get_running_kernel_pkg():
+    """Return the package of the currently running kernel, needed to force
+    assignment for 'kernel' package to a real one"""
+
+    system = platform.system()
+    release = platform.release()
+
+    if system == 'Linux':
+        return 'linux-image-' + release
+    elif system == 'GNU/kFreeBSD':
+        return 'kfreebsd-image-' + release
+    else:
+        return None
+
+def exec_and_parse_bugscript(handler, bugscript):
+    """Execute and parse the output of the package bugscript, in particular
+    identifying the headers and pseudo-headers blocks, if present"""
+
+    fh, filename = TempFile()
+    fh.close()
+    rc = os.system('LC_ALL=C %s %s %s' % (handler, commands.mkarg(bugscript),
+                                          commands.mkarg(filename)))
+
+    isheaders = False
+    ispseudoheaders = False
+    headers = pseudoheaders = text = ''
+    fp = open(filename)
+    for line in fp.readlines():
+        # we identify the blocks for headers and pseudo-h
+        if line == '-- BEGIN HEADERS --\n':
+            isheaders = True
+        elif line == '-- END HEADERS --\n':
+            isheaders = False
+        elif line == '-- BEGIN PSEUDOHEADERS --\n':
+            ispseudoheaders = True
+        elif line == '-- END PSEUDOHEADERS --\n':
+            ispseudoheaders = False
+        else:
+            if isheaders:
+                headers += line
+            elif ispseudoheaders:
+                pseudoheaders += line
+            else:
+                text += line
+    fp.close()
+    cleanup_temp_file(filename)
+
+    text = text.decode('utf-8', 'replace')
+    return (rc, headers, pseudoheaders, text)
